import * as i0 from '@angular/core';
import { InjectionToken, Injectable, inject, Inject, EventEmitter, Component, ChangeDetectionStrategy, Input, Output, ElementRef, PLATFORM_ID, HostListener, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { formatDate, getLocaleDayNames, FormStyle, TranslationWidth, isPlatformBrowser, CommonModule } from '@angular/common';
import { BehaviorSubject, Subject, takeUntil, interval } from 'rxjs';
import { startWith } from 'rxjs/operators';
import { __decorate } from 'tslib';
import * as i2 from 'angular-resizable-element';
import { ResizableModule } from 'angular-resizable-element';
import * as i3 from 'angular-draggable-droppable';
import { DragAndDropModule } from 'angular-draggable-droppable';

class DateHelpers {
    static generateDateId(date) {
        return `${date.getFullYear()}-${date.getMonth()}-${date.getDate()}-${date.getHours()}-${date.getMinutes()}`;
    }
    static lastDayOfMonth(date) {
        const dateWithLastDayOfMonth = new Date(date);
        dateWithLastDayOfMonth.setMonth(dateWithLastDayOfMonth.getMonth() + 1);
        dateWithLastDayOfMonth.setDate(0);
        return dateWithLastDayOfMonth;
    }
    static getDaysInMonth(date) {
        return new Date(date.getFullYear(), date.getMonth() + 1, 0).getDate();
    }
    static firstMondayOfMonth(date) {
        const firstDay = new Date(new Date(date).setDate(1));
        const monday = DateHelpers.firstDayOfWeek(firstDay);
        return monday.getMonth() === date.getMonth() ? monday : new Date(monday.setDate(monday.getDate() + 7));
    }
    static firstDayOfWeek(date) {
        date = new Date(date);
        const first = date.getDate() - date.getDay() + 1;
        return new Date(new Date(date).setDate(first));
    }
    static lastDayOfWeek(date) {
        date = new Date(date);
        const dayOfWeek = date.getDay();
        const diffToSunday = (dayOfWeek === 0) ? 0 : 7 - dayOfWeek;
        date.setDate(date.getDate() + diffToSunday);
        return date;
    }
    static dayBeginningTime(day) {
        day = new Date(day);
        day.setHours(0, 0, 0, 0);
        return day;
    }
    static dayEndingTime(day) {
        day = new Date(day);
        day.setHours(23, 59, 59, 999);
        return day;
    }
}
var MillisecondsToTime;
(function (MillisecondsToTime) {
    MillisecondsToTime[MillisecondsToTime["Minute"] = 60000] = "Minute";
    MillisecondsToTime[MillisecondsToTime["Day"] = 86400000] = "Day";
    MillisecondsToTime[MillisecondsToTime["Week"] = 604800000] = "Week";
})(MillisecondsToTime || (MillisecondsToTime = {}));

class ItemsIterator {
    constructor() {
        this._items = [];
    }
    get items() {
        return this._items;
    }
    setItems(items) {
        this._items = items;
        this._validate();
        this._createItemsLevels();
    }
    isEmpty() {
        return !this._items?.length;
    }
    getFirstItem(onlyVisible) {
        let firstItem = null;
        this.forEach((item, parent) => {
            if (!item.startDate || !item.endDate) {
                return;
            }
            if (!firstItem || new Date(firstItem.startDate).getTime() > new Date(item.startDate).getTime()) {
                firstItem = item;
            }
        }, onlyVisible);
        return firstItem;
    }
    getLastItem(onlyVisible) {
        let lastItem = null;
        this.forEach((item, parent) => {
            if (!item.startDate || !item.endDate) {
                return;
            }
            if (!lastItem || new Date(lastItem.endDate).getTime() < new Date(item.endDate).getTime()) {
                lastItem = item;
            }
        }, onlyVisible);
        return lastItem;
    }
    forEach(handler, onlyVisible = false) {
        function iterateAll(items, parent) {
            (items ?? []).forEach(item => {
                handler(item, parent);
                iterateAll(item.streamItems ?? [], item);
                if (!onlyVisible || item.childrenItemsExpanded) {
                    iterateAll(item.childrenItems ?? [], item);
                }
            });
        }
        iterateAll(this._items, null);
    }
    _createItemsLevels() {
        this.forEach((item, parent) => {
            if (item.streamItems) {
                item._streamLevels = this._createItemLevels(item);
            }
        });
    }
    _createItemLevels(item) {
        const levels = [];
        item.streamItems.forEach(item => {
            let isLevelFound = false;
            let currentLevelIndex = 0;
            while (!isLevelFound) {
                const levelItems = levels[currentLevelIndex];
                if (!levelItems) {
                    levels[currentLevelIndex] = [item];
                    isLevelFound = true;
                    break;
                }
                const isItemCollides = levelItems.some(levelItem => this._isItemsCollides(levelItem, item));
                if (!isItemCollides) {
                    levels[currentLevelIndex].push(item);
                    isLevelFound = true;
                    break;
                }
                currentLevelIndex++;
            }
        });
        return levels;
    }
    _isItemsCollides(item1, item2) {
        const item1Start = item1._left;
        const item1End = item1._left + item1._width;
        const item2Start = item2._left;
        const item2End = item2._left + item2._width;
        return item1Start === item2Start || item1End === item2End ||
            item1End > item2Start && item1Start < item2End ||
            item2End > item1Start && item2Start < item1End;
    }
    _validate() {
        this.forEach((item) => {
            if ((item.startDate && !item.endDate) || (item.endDate && !item.startDate)) {
                this._removeItemDates(item);
            }
            if (item.streamItems) {
                this._removeItemDates(item);
            }
        });
    }
    _removeItemDates(item) {
        delete item.startDate;
        delete item.endDate;
    }
}

class ZoomsHandler {
    get activeZoom() {
        return this._activeZoom$.value;
    }
    get zooms() {
        return this._zooms;
    }
    constructor(zooms) {
        this._activeZoom$ = new BehaviorSubject(null);
        this.activeZoom$ = this._activeZoom$.asObservable();
        this.setZooms(zooms);
    }
    setZooms(zooms) {
        this._zooms = (zooms ?? []).map((item, index) => ({ ...item, index }));
        this._activeZoom$.next(this.getLastZoom());
    }
    getFirstZoom() {
        return this._zooms[0];
    }
    getLastZoom() {
        return this._zooms[this._zooms.length - 1];
    }
    zoomIn() {
        let newZoomIndex = this.activeZoom.index + 1;
        const lastZoomIndex = this.getLastZoom().index;
        if (newZoomIndex > lastZoomIndex) {
            newZoomIndex = lastZoomIndex;
        }
        this.changeActiveZoom(this._zooms[newZoomIndex]);
    }
    zoomOut() {
        let newZoomIndex = this.activeZoom.index - 1;
        const firstZoomIndex = this.getFirstZoom().index;
        if (newZoomIndex < firstZoomIndex) {
            newZoomIndex = firstZoomIndex;
        }
        this.changeActiveZoom(this._zooms[newZoomIndex]);
    }
    changeActiveZoom(zoom) {
        if (zoom) {
            this._activeZoom$.next(this._zooms[this._findZoomIndex(zoom)]);
        }
    }
    isZoomActive(zoom) {
        return this._findZoomIndex(zoom) === this.activeZoom.index;
    }
    _findZoomIndex(zoom) {
        return this._zooms.findIndex(i => i.columnWidth === zoom.columnWidth && i.viewMode === zoom.viewMode);
    }
}

var TimelineViewMode;
(function (TimelineViewMode) {
    TimelineViewMode[TimelineViewMode["Month"] = 101] = "Month";
    TimelineViewMode[TimelineViewMode["Week"] = 102] = "Week";
    TimelineViewMode[TimelineViewMode["Day"] = 103] = "Day";
})(TimelineViewMode || (TimelineViewMode = {}));

const ZOOMS = new InjectionToken('Zooms');
const DefaultZooms = [
    { columnWidth: 45, viewMode: TimelineViewMode.Month },
    { columnWidth: 60, viewMode: TimelineViewMode.Month },
    { columnWidth: 80, viewMode: TimelineViewMode.Month },
    { columnWidth: 110, viewMode: TimelineViewMode.Month },
    { columnWidth: 140, viewMode: TimelineViewMode.Month },
    { columnWidth: 200, viewMode: TimelineViewMode.Month },
    { columnWidth: 240, viewMode: TimelineViewMode.Month },
    { columnWidth: 60, viewMode: TimelineViewMode.Week },
    { columnWidth: 80, viewMode: TimelineViewMode.Week },
    { columnWidth: 110, viewMode: TimelineViewMode.Week },
    { columnWidth: 140, viewMode: TimelineViewMode.Week },
    { columnWidth: 200, viewMode: TimelineViewMode.Week },
    { columnWidth: 240, viewMode: TimelineViewMode.Week },
    { columnWidth: 45, viewMode: TimelineViewMode.Day },
    { columnWidth: 60, viewMode: TimelineViewMode.Day },
    { columnWidth: 80, viewMode: TimelineViewMode.Day },
    { columnWidth: 110, viewMode: TimelineViewMode.Day },
    { columnWidth: 140, viewMode: TimelineViewMode.Day },
    { columnWidth: 200, viewMode: TimelineViewMode.Day },
    { columnWidth: 240, viewMode: TimelineViewMode.Day },
];

class RowDeterminant {
    constructor(_itemsIterator) {
        this._itemsIterator = _itemsIterator;
        this._generateMap();
    }
    _generateMap() {
        const map = [];
        const iterate = (items) => {
            (items ?? []).forEach(item => {
                if (item.streamItems) {
                    item._streamLevels.forEach((levelArr, index) => {
                        map.push({ stream: item, items: levelArr });
                    });
                }
                else {
                    map.push({ stream: item, items: [item] });
                }
                if (item.childrenItemsExpanded) {
                    iterate(item.childrenItems ?? []);
                }
            });
        };
        iterate(this._itemsIterator.items);
        this.rows = map;
    }
    getRowIndexByItem(item) {
        let index;
        for (let i = 0; i < this.rows.length; i++) {
            const group = this.rows[i];
            if (item.id === group.stream.id) {
                index = i;
                break;
            }
            const hasChild = group.items.find(i => i.id === item.id);
            if (hasChild) {
                index = i;
            }
        }
        return index;
    }
    getStreamByRowIndex(index) {
        return this.rows[index]?.stream;
    }
}

function DatesCacheDecorator() {
    return function (target, methodName, descriptor) {
        if (!target.__datesCache) {
            target.__datesCache = new Map();
        }
        const originalMethod = descriptor.value;
        descriptor.value = function (...args) {
            const cacheKey = `${methodName}-${[...args].map(date => DateHelpers.generateDateId(date)).join('-')}`;
            if (target.__datesCache.has(cacheKey)) {
                return target.__datesCache.get(cacheKey);
            }
            const result = originalMethod.apply(this, args);
            target.__datesCache.set(cacheKey, result);
            return result;
        };
    };
}

class BaseScaleGenerator {
    constructor() {
        this._config = this._getConfig();
        this.formatter = this._config.formatter;
    }
    getStartDate(itemsBuilder) {
        if (this._config.getStartDate) {
            return this._config.getStartDate(itemsBuilder);
        }
        const firstItem = itemsBuilder.getFirstItem(false);
        const now = Date.now();
        const firstItemTime = new Date(firstItem?.startDate ?? now).getTime();
        return this._validateStartDate(firstItemTime < now ? firstItemTime : now);
    }
    getEndDate(itemsBuilder) {
        if (this._config.getEndDate) {
            return this._config.getEndDate(itemsBuilder);
        }
        const lastItem = itemsBuilder.getLastItem(false);
        const now = Date.now();
        const lastItemDate = new Date(lastItem?.endDate ?? now);
        return this._validateEndDate(lastItemDate.getTime() < now ? now : lastItemDate);
    }
    generateScale(startDate, endDate) {
        let currentDate = new Date(startDate);
        const endTime = endDate.getTime();
        const columns = [];
        while (currentDate.getTime() <= endTime) {
            const date = new Date(currentDate);
            columns.push({
                id: DateHelpers.generateDateId(date),
                date: date,
                index: this._getColumnIndex(date),
                groups: this._generateGroups(date),
            });
            currentDate = this._getNextColumnDate(currentDate);
        }
        return {
            startDate,
            endDate,
            columns
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BaseScaleGenerator, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BaseScaleGenerator }); }
}
__decorate([
    DatesCacheDecorator()
], BaseScaleGenerator.prototype, "generateScale", null);
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BaseScaleGenerator, decorators: [{
            type: Injectable
        }], ctorParameters: () => [], propDecorators: { generateScale: [] } });

class DayScaleFormatter {
    formatColumn(column, columnWidth, locale) {
        if (columnWidth < 65)
            return formatDate(column.date, 'dd', locale);
        if (columnWidth > 180)
            return formatDate(column.date, 'EEEE dd/MM', locale);
        return formatDate(column.date, 'EEE dd/MM', locale);
    }
    formatGroup(group, locale) {
        return formatDate(group.date, 'LLLL', locale);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleFormatter, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleFormatter }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleFormatter, decorators: [{
            type: Injectable
        }] });

const DAY_SCALE_GENERATOR_CONFIG = new InjectionToken('Day scale config');
const DefaultConfig$2 = {
    formatter: new DayScaleFormatter(),
};
class DefaultDayScaleGenerator extends BaseScaleGenerator {
    _getConfig() {
        return { ...DefaultConfig$2, ...inject(DAY_SCALE_GENERATOR_CONFIG, {}) };
    }
    _validateStartDate(startDate) {
        const countOfEmptyMonthsBefore = 1;
        startDate = new Date(startDate);
        startDate.setDate(1);
        startDate = DateHelpers.dayBeginningTime(startDate);
        startDate.setMonth(startDate.getMonth() - countOfEmptyMonthsBefore);
        return startDate;
    }
    _validateEndDate(endDate) {
        const countOfEmptyMonthsAfter = 1;
        endDate = new Date(endDate);
        return new Date(DateHelpers.lastDayOfMonth(endDate).setMonth(endDate.getMonth() + countOfEmptyMonthsAfter));
    }
    _generateGroups(date) {
        date = new Date(date.getFullYear(), date.getMonth(), 1, 0, 0, 0, 0);
        return [{ date, id: DateHelpers.generateDateId(date), coverageInPercents: 100 }];
    }
    _getColumnIndex(date) {
        return date.getDate();
    }
    _getNextColumnDate(date) {
        return new Date(date.setDate(date.getDate() + 1));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultDayScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultDayScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultDayScaleGenerator, decorators: [{
            type: Injectable
        }] });
class DayScaleGenerator extends DefaultDayScaleGenerator {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DayScaleGenerator, decorators: [{
            type: Injectable
        }] });

class WeekScaleFormatter {
    formatColumn(column, columnWidth, locale) {
        if (columnWidth > 100) {
            const days = getLocaleDayNames(locale, FormStyle.Format, TranslationWidth.Abbreviated);
            return `${days[1]}-${days[0]} (${column.index})`;
        }
        return String(column.index);
    }
    formatGroup(group, locale) {
        return formatDate(group.date, 'LLLL y', locale);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleFormatter, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleFormatter }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleFormatter, decorators: [{
            type: Injectable
        }] });

const WEEK_SCALE_GENERATOR_CONFIG = new InjectionToken('Week scale config');
const DefaultConfig$1 = {
    formatter: new WeekScaleFormatter(),
};
class DefaultWeekScaleGenerator extends BaseScaleGenerator {
    _getConfig() {
        return { ...DefaultConfig$1, ...inject(WEEK_SCALE_GENERATOR_CONFIG, {}) };
    }
    _validateStartDate(startDate) {
        const countOfEmptyMonthsBefore = 1;
        const newDate = new Date(startDate);
        newDate.setMonth(newDate.getMonth() - countOfEmptyMonthsBefore);
        return DateHelpers.firstMondayOfMonth(newDate);
    }
    _validateEndDate(endDate) {
        const countOfEmptyMonthsAfter = 1;
        const newDate = new Date(endDate);
        newDate.setMonth(newDate.getMonth() + countOfEmptyMonthsAfter);
        return DateHelpers.lastDayOfWeek(newDate);
    }
    _generateGroups(date) {
        const weekStart = DateHelpers.firstDayOfWeek(date);
        const weekEnd = DateHelpers.lastDayOfWeek(date);
        const weekRelatedToTwoMonths = weekStart.getMonth() !== weekEnd.getMonth();
        const weekStartGroupDate = new Date(weekStart.getFullYear(), weekStart.getMonth(), 1, 0, 0, 0, 0);
        const groups = [
            { date: weekStartGroupDate, id: DateHelpers.generateDateId(weekStartGroupDate), coverageInPercents: 100 }
        ];
        if (weekRelatedToTwoMonths) {
            groups[0].coverageInPercents = (DateHelpers.getDaysInMonth(weekStart) - (weekStart.getDate() - 1)) / 7 * 100;
            const weekEndGroupDate = new Date(weekEnd.getFullYear(), weekEnd.getMonth(), 1, 0, 0, 0, 0);
            groups.push({
                date: weekEndGroupDate,
                id: DateHelpers.generateDateId(weekEndGroupDate),
                coverageInPercents: 100 - groups[0].coverageInPercents
            });
        }
        return groups;
    }
    _getColumnIndex(date) {
        const weekMonday = DateHelpers.firstDayOfWeek(date);
        return Math.ceil(weekMonday.getDate() / 7);
    }
    _getNextColumnDate(date) {
        return new Date(date.setDate(date.getDate() + 7));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultWeekScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultWeekScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultWeekScaleGenerator, decorators: [{
            type: Injectable
        }] });
class WeekScaleGenerator extends DefaultWeekScaleGenerator {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: WeekScaleGenerator, decorators: [{
            type: Injectable
        }] });

class MonthScaleFormatter {
    formatColumn(column, columnWidth, locale) {
        if (columnWidth < 65)
            return String(column.index);
        if (columnWidth > 180)
            return formatDate(column.date, 'LLLL', locale);
        return formatDate(column.date, 'LLL', locale);
    }
    formatGroup(group, locale) {
        return String(group.date.getFullYear());
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleFormatter, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleFormatter }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleFormatter, decorators: [{
            type: Injectable
        }] });

const MONTH_SCALE_GENERATOR_CONFIG = new InjectionToken('Month scale config');
const DefaultConfig = {
    formatter: new MonthScaleFormatter(),
};
class DefaultMonthScaleGenerator extends BaseScaleGenerator {
    _getConfig() {
        return { ...DefaultConfig, ...inject(MONTH_SCALE_GENERATOR_CONFIG, {}) };
    }
    _validateStartDate(startDate) {
        const newDate = new Date(startDate);
        const countOfEmptyYearsBefore = 1;
        newDate.setDate(1);
        newDate.setMonth(0);
        newDate.setFullYear(newDate.getFullYear() - countOfEmptyYearsBefore);
        return newDate;
    }
    _validateEndDate(endDate) {
        const newDate = DateHelpers.lastDayOfMonth(endDate);
        const countOfEmptyYearsAfter = 1;
        newDate.setMonth(11);
        newDate.setFullYear(newDate.getFullYear() + countOfEmptyYearsAfter);
        return newDate;
    }
    _generateGroups(date) {
        date = new Date(date.getFullYear(), 1, 0, 0, 0, 0, 0);
        return [{ date, id: DateHelpers.generateDateId(date), coverageInPercents: 100 }];
    }
    _getColumnIndex(date) {
        return date.getMonth() + 1;
    }
    _getNextColumnDate(date) {
        return new Date(date.setMonth(date.getMonth() + 1));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultMonthScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultMonthScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultMonthScaleGenerator, decorators: [{
            type: Injectable
        }] });
class MonthScaleGenerator extends DefaultMonthScaleGenerator {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleGenerator, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleGenerator }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: MonthScaleGenerator, decorators: [{
            type: Injectable
        }] });

class BaseViewModeAdaptor {
    getMiddleDate(startDate, endDate) {
        const uniqueColumns = this.getUniqueColumnsWithinRange(startDate, endDate);
        return this.addColumnToDate(this.getBeginningDateOfColumn(startDate), uniqueColumns / 2);
    }
}

class DaysViewModeAdaptor extends BaseViewModeAdaptor {
    getUniqueColumnsWithinRange(start, end) {
        const startDate = new Date(start.getFullYear(), start.getMonth(), start.getDate());
        const endDate = new Date(end.getFullYear(), end.getMonth(), end.getDate());
        return Math.round(Math.abs((startDate.getTime() - endDate.getTime()) / MillisecondsToTime.Day)) + 1;
    }
    getDurationInColumns(startDate, endDate) {
        return Math.abs((startDate.getTime() - endDate.getTime()) / MillisecondsToTime.Day);
    }
    addColumnToDate(date, days) {
        const newDate = new Date(date);
        newDate.setDate(date.getDate() + days);
        newDate.setHours(newDate.getHours() + ((days % 1) * 24));
        return newDate;
    }
    getEndingDateOfColumn(date) {
        return DateHelpers.dayEndingTime(date);
    }
    getBeginningDateOfColumn(date) {
        return DateHelpers.dayBeginningTime(date);
    }
}
__decorate([
    DatesCacheDecorator()
], DaysViewModeAdaptor.prototype, "getUniqueColumnsWithinRange", null);
__decorate([
    DatesCacheDecorator()
], DaysViewModeAdaptor.prototype, "getDurationInColumns", null);

class WeeksViewModeAdaptor extends BaseViewModeAdaptor {
    getUniqueColumnsWithinRange(start, end) {
        const monday = DateHelpers.firstDayOfWeek(start);
        const last = DateHelpers.lastDayOfWeek(end);
        return Math.round(this.getDurationInColumns(monday, last));
    }
    getDurationInColumns(startDate, endDate) {
        return Math.abs((startDate.getTime() - endDate.getTime()) / MillisecondsToTime.Week);
    }
    addColumnToDate(date, weeks) {
        const newDate = new Date(date);
        newDate.setDate(date.getDate() + (7 * weeks));
        newDate.setHours(newDate.getHours() + (((weeks / 7) % 1) * 24));
        return newDate;
    }
    getBeginningDateOfColumn(date) {
        const start = DateHelpers.firstDayOfWeek(new Date(date));
        return DateHelpers.dayBeginningTime(start);
    }
    getEndingDateOfColumn(date) {
        const end = DateHelpers.lastDayOfWeek(new Date(date));
        return DateHelpers.dayEndingTime(end);
    }
}
__decorate([
    DatesCacheDecorator()
], WeeksViewModeAdaptor.prototype, "getUniqueColumnsWithinRange", null);
__decorate([
    DatesCacheDecorator()
], WeeksViewModeAdaptor.prototype, "getDurationInColumns", null);

class MonthsViewModeAdaptor extends BaseViewModeAdaptor {
    getBeginningDateOfColumn(date) {
        const start = new Date(date);
        start.setDate(1);
        return DateHelpers.dayBeginningTime(start);
    }
    getEndingDateOfColumn(date) {
        const end = new Date(date);
        end.setDate(DateHelpers.lastDayOfMonth(date).getDate());
        return DateHelpers.dayEndingTime(end);
    }
    getUniqueColumnsWithinRange(startDate, endDate) {
        const diff = this._getCountOfFullMonths(startDate, endDate);
        return (diff < 0 ? 0 : diff) + 1;
    }
    getDurationInColumns(startDate, endDate) {
        const diff = this._getCountOfFullMonths(startDate, endDate);
        const firstMonthCompletedPercent = ((startDate.getDate() - 1) + (startDate.getHours() / 24)) / DateHelpers.getDaysInMonth(startDate);
        const secondMonthCompletedPercent = ((endDate.getDate() - 1) + (endDate.getHours() / 24)) / DateHelpers.getDaysInMonth(endDate);
        return diff - firstMonthCompletedPercent + secondMonthCompletedPercent;
    }
    addColumnToDate(date, months) {
        const newDate = new Date(date);
        newDate.setMonth(date.getMonth() + months);
        const days = DateHelpers.getDaysInMonth(newDate) * (months % 1);
        newDate.setDate(newDate.getDate() + days);
        newDate.setHours(newDate.getHours() + ((days % 1) * 24));
        return newDate;
    }
    _getCountOfFullMonths(startDate, endDate) {
        const yearsDiff = endDate.getFullYear() - startDate.getFullYear();
        const startMonth = startDate.getMonth();
        const endMonth = endDate.getMonth() + (12 * yearsDiff);
        return endMonth - startMonth;
    }
}
__decorate([
    DatesCacheDecorator()
], MonthsViewModeAdaptor.prototype, "getUniqueColumnsWithinRange", null);
__decorate([
    DatesCacheDecorator()
], MonthsViewModeAdaptor.prototype, "getDurationInColumns", null);

class DefaultStrategyManager {
    constructor(_dayGenerator, _weekGenerator, _monthGenerator) {
        this._dayGenerator = _dayGenerator;
        this._weekGenerator = _weekGenerator;
        this._monthGenerator = _monthGenerator;
        this._generatorsDictionary = {
            [TimelineViewMode.Day]: this._dayGenerator,
            [TimelineViewMode.Week]: this._weekGenerator,
            [TimelineViewMode.Month]: this._monthGenerator,
        };
        this._calculatorsDictionary = {
            [TimelineViewMode.Day]: new DaysViewModeAdaptor(),
            [TimelineViewMode.Week]: new WeeksViewModeAdaptor(),
            [TimelineViewMode.Month]: new MonthsViewModeAdaptor(),
        };
    }
    getViewModeAdaptor(viewMode) {
        return this._calculatorsDictionary[viewMode];
    }
    getScaleGenerator(viewMode) {
        return this._generatorsDictionary[viewMode];
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultStrategyManager, deps: [{ token: DayScaleGenerator }, { token: WeekScaleGenerator }, { token: MonthScaleGenerator }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultStrategyManager }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DefaultStrategyManager, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [DayScaleGenerator]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [WeekScaleGenerator]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [MonthScaleGenerator]
                }] }] });
class StrategyManager extends DefaultStrategyManager {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: StrategyManager, deps: null, target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: StrategyManager }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: StrategyManager, decorators: [{
            type: Injectable
        }] });

class TimelineItemComponent {
    set item(item) {
        this._item = item;
        item.updateView = () => this._cdr.detectChanges();
        this._checkIsInScaleRange();
    }
    ;
    set scale(scale) {
        this._scale = scale;
        this._checkIsInScaleRange();
    }
    ;
    get item() {
        return this._item;
    }
    constructor(_cdr, _renderer) {
        this._cdr = _cdr;
        this._renderer = _renderer;
        this.isInScaleRange = true;
        this.isItemResizingStarted = false;
        this.itemResized = new EventEmitter();
        this.itemMoved = new EventEmitter();
    }
    onItemResizeStart(event) {
        this.isItemResizingStarted = true;
        this._cdr.markForCheck();
    }
    onItemResizeEnd(event) {
        this.itemResized.emit({ event, item: this._item });
        setTimeout(() => this.isItemResizingStarted = false);
    }
    onItemDragStart(event) {
        this._setRowZIndex(1000);
    }
    onItemDropped(event) {
        if (!this.isItemResizingStarted) {
            this.itemMoved.emit({ event, item: this._item });
        }
        this._setRowZIndex(1);
    }
    _checkIsInScaleRange() {
        if (!this._item || !this._scale) {
            return;
        }
        if (!this._item.startDate || !this._item.endDate) {
            this.isInScaleRange = true;
            this._cdr.markForCheck();
            return;
        }
        this.isInScaleRange = this._scale.startDate.getTime() <= this._item.startDate.getTime()
            && this._scale.endDate.getTime() >= this._item.endDate.getTime();
        this._cdr.markForCheck();
    }
    _setRowZIndex(index) {
        this._renderer.setStyle(this.rowContainer, 'z-index', index);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineItemComponent, deps: [{ token: i0.ChangeDetectorRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: TimelineItemComponent, selector: "timeline-item", inputs: { item: "item", scale: "scale", rowContainer: "rowContainer", height: "height", rowHeight: "rowHeight", locale: "locale", contentTemplate: "contentTemplate" }, outputs: { itemResized: "itemResized", itemMoved: "itemMoved" }, ngImport: i0, template: "<div class=\"timeline-item\"\r\n     *ngIf=\"isInScaleRange\"\r\n     ghostElementPositioning='absolute'\r\n     mwlResizable\r\n     mwlDraggable\r\n     [dragActiveClass]=\"'timeline-item_dragging'\"\r\n     (dragStart)='onItemDragStart($event)'\r\n     (dragEnd)='onItemDropped($event)'\r\n     (resizeEnd)='onItemResizeEnd($event)'\r\n     (resizeStart)='onItemResizeStart($event)'\r\n     [dragAxis]=\"{y: item.canDragY, x: item.canDragX}\"\r\n     [enableGhostResize]='true'\r\n     [dragSnapGrid]=\"{y: rowHeight}\"\r\n     [style.height.px]='height'\r\n     [style.left.px]='item?._left'\r\n     [style.width.px]='item?._width'>\r\n  <div class=\"item-custom-template\" *ngIf=\"contentTemplate; else nameTemplate\">\r\n    <ng-container [ngTemplateOutlet]=\"contentTemplate\"\r\n                  [ngTemplateOutletContext]=\"{$implicit: item, locale: locale}\"\r\n    ></ng-container>\r\n  </div>\r\n\r\n  <ng-template #nameTemplate>\r\n   <div class=\"default-content\">\r\n     {{item.name}}\r\n   </div>\r\n  </ng-template>\r\n\r\n  <div *ngIf='item.canResizeLeft'\r\n       [resizeEdges]='{ left: item.canResizeLeft }'\r\n       class='resize-handle-left'\r\n       mwlResizeHandle\r\n  ></div>\r\n  <div *ngIf='item.canResizeRight'\r\n       [resizeEdges]='{ right: item.canResizeRight }'\r\n       class='resize-handle-right'\r\n       mwlResizeHandle\r\n  ></div>\r\n</div>\r\n", styles: [".timeline-item{height:100%;position:absolute;box-sizing:border-box;overflow:hidden;top:5px}.item-custom-template{width:100%;height:100%;position:relative;box-sizing:border-box}.default-content{height:100%;width:100%;background-color:#098ed2;padding:2px 10px;display:flex;align-items:center;color:#f1f1f1;box-sizing:border-box;border-radius:2px;cursor:pointer}mwlResizable{box-sizing:border-box}.resize-handle-left,.resize-handle-right{position:absolute;height:100%;cursor:col-resize;width:5px;top:0}.resize-handle-left{left:0}.resize-handle-right{right:0}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i2.ResizableDirective, selector: "[mwlResizable]", inputs: ["validateResize", "enableGhostResize", "resizeSnapGrid", "resizeCursors", "ghostElementPositioning", "allowNegativeResizes", "mouseMoveThrottleMS"], outputs: ["resizeStart", "resizing", "resizeEnd"], exportAs: ["mwlResizable"] }, { kind: "directive", type: i2.ResizeHandleDirective, selector: "[mwlResizeHandle]", inputs: ["resizeEdges", "resizableContainer"] }, { kind: "directive", type: i3.DraggableDirective, selector: "[mwlDraggable]", inputs: ["dropData", "dragAxis", "dragSnapGrid", "ghostDragEnabled", "showOriginalElementWhileDragging", "validateDrag", "dragCursor", "dragActiveClass", "ghostElementAppendTo", "ghostElementTemplate", "touchStartLongPress", "autoScroll"], outputs: ["dragPointerDown", "dragStart", "ghostElementCreated", "dragging", "dragEnd"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineItemComponent, decorators: [{
            type: Component,
            args: [{ selector: 'timeline-item', changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class=\"timeline-item\"\r\n     *ngIf=\"isInScaleRange\"\r\n     ghostElementPositioning='absolute'\r\n     mwlResizable\r\n     mwlDraggable\r\n     [dragActiveClass]=\"'timeline-item_dragging'\"\r\n     (dragStart)='onItemDragStart($event)'\r\n     (dragEnd)='onItemDropped($event)'\r\n     (resizeEnd)='onItemResizeEnd($event)'\r\n     (resizeStart)='onItemResizeStart($event)'\r\n     [dragAxis]=\"{y: item.canDragY, x: item.canDragX}\"\r\n     [enableGhostResize]='true'\r\n     [dragSnapGrid]=\"{y: rowHeight}\"\r\n     [style.height.px]='height'\r\n     [style.left.px]='item?._left'\r\n     [style.width.px]='item?._width'>\r\n  <div class=\"item-custom-template\" *ngIf=\"contentTemplate; else nameTemplate\">\r\n    <ng-container [ngTemplateOutlet]=\"contentTemplate\"\r\n                  [ngTemplateOutletContext]=\"{$implicit: item, locale: locale}\"\r\n    ></ng-container>\r\n  </div>\r\n\r\n  <ng-template #nameTemplate>\r\n   <div class=\"default-content\">\r\n     {{item.name}}\r\n   </div>\r\n  </ng-template>\r\n\r\n  <div *ngIf='item.canResizeLeft'\r\n       [resizeEdges]='{ left: item.canResizeLeft }'\r\n       class='resize-handle-left'\r\n       mwlResizeHandle\r\n  ></div>\r\n  <div *ngIf='item.canResizeRight'\r\n       [resizeEdges]='{ right: item.canResizeRight }'\r\n       class='resize-handle-right'\r\n       mwlResizeHandle\r\n  ></div>\r\n</div>\r\n", styles: [".timeline-item{height:100%;position:absolute;box-sizing:border-box;overflow:hidden;top:5px}.item-custom-template{width:100%;height:100%;position:relative;box-sizing:border-box}.default-content{height:100%;width:100%;background-color:#098ed2;padding:2px 10px;display:flex;align-items:center;color:#f1f1f1;box-sizing:border-box;border-radius:2px;cursor:pointer}mwlResizable{box-sizing:border-box}.resize-handle-left,.resize-handle-right{position:absolute;height:100%;cursor:col-resize;width:5px;top:0}.resize-handle-left{left:0}.resize-handle-right{right:0}\n"] }]
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }, { type: i0.Renderer2 }], propDecorators: { item: [{
                type: Input
            }], scale: [{
                type: Input
            }], rowContainer: [{
                type: Input
            }], height: [{
                type: Input
            }], rowHeight: [{
                type: Input
            }], locale: [{
                type: Input
            }], contentTemplate: [{
                type: Input
            }], itemResized: [{
                type: Output
            }], itemMoved: [{
                type: Output
            }] } });

class TimelineDateMarkerComponent {
    set scale(scale) {
        this._checkIsInScaleRange(scale);
    }
    ;
    constructor(_cdr) {
        this._cdr = _cdr;
        this.isInScaleRange = true;
        this.leftPosition = 0;
    }
    _checkIsInScaleRange(scale) {
        const now = Date.now();
        this.isInScaleRange = scale.startDate.getTime() <= now && scale.endDate.getTime() >= now;
        this._cdr.detectChanges();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineDateMarkerComponent, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: TimelineDateMarkerComponent, selector: "timeline-date-marker", inputs: { leftPosition: "leftPosition", headerHeight: "headerHeight", customTemplate: "customTemplate", scale: "scale" }, ngImport: i0, template: "<ng-container *ngIf=\"isInScaleRange\">\r\n  <ng-container *ngIf=\"customTemplate; else defaultDateMarkerTemplate\"\r\n                [ngTemplateOutlet]=\"customTemplate\"\r\n                [ngTemplateOutletContext]=\"{leftPosition: leftPosition}\">\r\n  </ng-container>\r\n\r\n  <ng-template #defaultDateMarkerTemplate>\r\n    <div [style.left.px]=\"leftPosition\"\r\n         [style.height]=\"'calc(100% - ' + headerHeight + 'px)'\"\r\n         class='date-marker'></div>\r\n  </ng-template>\r\n</ng-container>\r\n", styles: [".date-marker{position:absolute;width:2px;height:100%;background-color:#2ac226;z-index:2;display:flex;flex-direction:column;transform:translate(-50%)}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineDateMarkerComponent, decorators: [{
            type: Component,
            args: [{ selector: 'timeline-date-marker', changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-container *ngIf=\"isInScaleRange\">\r\n  <ng-container *ngIf=\"customTemplate; else defaultDateMarkerTemplate\"\r\n                [ngTemplateOutlet]=\"customTemplate\"\r\n                [ngTemplateOutletContext]=\"{leftPosition: leftPosition}\">\r\n  </ng-container>\r\n\r\n  <ng-template #defaultDateMarkerTemplate>\r\n    <div [style.left.px]=\"leftPosition\"\r\n         [style.height]=\"'calc(100% - ' + headerHeight + 'px)'\"\r\n         class='date-marker'></div>\r\n  </ng-template>\r\n</ng-container>\r\n", styles: [".date-marker{position:absolute;width:2px;height:100%;background-color:#2ac226;z-index:2;display:flex;flex-direction:column;transform:translate(-50%)}\n"] }]
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }], propDecorators: { leftPosition: [{
                type: Input
            }], headerHeight: [{
                type: Input
            }], customTemplate: [{
                type: Input
            }], scale: [{
                type: Input
            }] } });

class TimelineScaleHeaderComponent {
    constructor() {
        this.groups = [];
    }
    get columns() {
        return this.scale?.columns ?? [];
    }
    ngOnChanges(changes) {
        this._generateGroups();
    }
    trackById(index, item) {
        return item.id;
    }
    _groupColumnGroups() {
        return this.scale.columns.reduce((groupsMap, column) => {
            column.groups.forEach(group => {
                groupsMap[group.id] = groupsMap[group.id] ?? [];
                groupsMap[group.id].push(group);
            });
            return groupsMap;
        }, {});
    }
    _generateGroups() {
        const groupedGroups = this._groupColumnGroups();
        this.groups = Object.keys(groupedGroups).map(groupId => ({
            id: groupId,
            name: this.formatter.formatGroup(groupedGroups[groupId][0], this.locale),
            width: groupedGroups[groupId].reduce((acc, curr) => acc + this.zoom.columnWidth * curr.coverageInPercents / 100, 0)
        }));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineScaleHeaderComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: TimelineScaleHeaderComponent, selector: "timeline-scale-header", inputs: { height: "height", scale: "scale", formatter: "formatter", locale: "locale", zoom: "zoom" }, usesOnChanges: true, ngImport: i0, template: "<div class='wrapper' [style.height.px]=\"height\">\r\n  <div class='groups' *ngIf=\"groups.length\">\r\n    <div *ngFor='let group of groups; trackBy: trackById; index as i'\r\n         [style.width.px]='group.width' class='group'>\r\n      <div>{{group.name}}</div>\r\n    </div>\r\n  </div>\r\n  <div class='columns'>\r\n    <div *ngFor='let column of columns; trackBy: trackById'>\r\n      <div class='column' [style.width.px]='zoom.columnWidth'>\r\n        {{formatter.formatColumn(column, zoom.columnWidth, locale)}}\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n\r\n", styles: [":host{z-index:9;position:sticky;top:0;display:flex;flex-direction:row;background:#f8f8f8}.wrapper{display:flex;flex-direction:column}.groups{display:flex;flex-grow:3}.groups .group{height:100%;display:flex;align-items:center;justify-content:center;border-bottom:1px solid #d0d0d0;position:relative;box-sizing:border-box;padding:0 4px}.groups .group:first-child:after{display:none}.groups .group:after{content:\"\";position:absolute;width:1px;height:100%;box-sizing:border-box;left:0;top:0;background:#d0d0d0}.columns{display:flex;flex-grow:2}.columns .column{height:100%;position:relative;display:flex;justify-content:center;align-items:center}.columns .column:after{content:\"\";width:1px;height:100%;left:100%;position:absolute;background:#d0d0d0}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineScaleHeaderComponent, decorators: [{
            type: Component,
            args: [{ selector: 'timeline-scale-header', changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class='wrapper' [style.height.px]=\"height\">\r\n  <div class='groups' *ngIf=\"groups.length\">\r\n    <div *ngFor='let group of groups; trackBy: trackById; index as i'\r\n         [style.width.px]='group.width' class='group'>\r\n      <div>{{group.name}}</div>\r\n    </div>\r\n  </div>\r\n  <div class='columns'>\r\n    <div *ngFor='let column of columns; trackBy: trackById'>\r\n      <div class='column' [style.width.px]='zoom.columnWidth'>\r\n        {{formatter.formatColumn(column, zoom.columnWidth, locale)}}\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n\r\n", styles: [":host{z-index:9;position:sticky;top:0;display:flex;flex-direction:row;background:#f8f8f8}.wrapper{display:flex;flex-direction:column}.groups{display:flex;flex-grow:3}.groups .group{height:100%;display:flex;align-items:center;justify-content:center;border-bottom:1px solid #d0d0d0;position:relative;box-sizing:border-box;padding:0 4px}.groups .group:first-child:after{display:none}.groups .group:after{content:\"\";position:absolute;width:1px;height:100%;box-sizing:border-box;left:0;top:0;background:#d0d0d0}.columns{display:flex;flex-grow:2}.columns .column{height:100%;position:relative;display:flex;justify-content:center;align-items:center}.columns .column:after{content:\"\";width:1px;height:100%;left:100%;position:absolute;background:#d0d0d0}\n"] }]
        }], propDecorators: { height: [{
                type: Input
            }], scale: [{
                type: Input
            }], formatter: [{
                type: Input
            }], locale: [{
                type: Input
            }], zoom: [{
                type: Input
            }] } });

class TimelinePanelComponent {
    constructor(_cdr) {
        this._cdr = _cdr;
        this.items = [];
        this.childGroupOffset = 15;
        this.widthChanged = new EventEmitter();
    }
    ngOnChanges(changes) {
        if (Object.keys(changes).some(key => ['width', 'minWidth', 'maxWidth'].includes(key))) {
            this._validateWidth();
        }
    }
    trackById(index, item) {
        return item.id;
    }
    handleResize(event) {
        const newWidth = event.rectangle.width;
        if (newWidth < this.minWidth || newWidth > this.maxWidth)
            return;
        this.width = newWidth;
        this.widthChanged.emit(this.width);
    }
    toggleExpand(item) {
        item.childrenItemsExpanded = !item.childrenItemsExpanded;
        this._cdr.markForCheck();
    }
    _validateWidth() {
        if (this.width < this.minWidth) {
            this.width = this.minWidth;
        }
        if (this.width > this.maxWidth) {
            this.width = this.maxWidth;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelinePanelComponent, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: TimelinePanelComponent, selector: "timeline-panel", inputs: { items: "items", label: "label", width: "width", resizable: "resizable", minWidth: "minWidth", maxWidth: "maxWidth", headerHeight: "headerHeight", rowHeight: "rowHeight", locale: "locale", childGroupOffset: "childGroupOffset", itemTemplate: "itemTemplate" }, outputs: { widthChanged: "widthChanged" }, usesOnChanges: true, ngImport: i0, template: "<div class='panel resize-handle-right'\r\n     mwlResizable\r\n     (resizing)='handleResize($event)'\r\n     [style.width.px]='width'>\r\n\r\n  <div mwlResizeHandle [resizeEdges]=\"{right: resizable}\"></div>\r\n\r\n  <div class='label' [style.height.px]=\"headerHeight\">{{label}}</div>\r\n\r\n  <ng-container *ngFor='let item of items; trackBy: trackById; let index = index'\r\n                [ngTemplateOutlet]='itemsIterationTemplate'\r\n                [ngTemplateOutletContext]='{item: item, index: index, depth: 0}'>\r\n  </ng-container>\r\n\r\n  <ng-template #itemsIterationTemplate let-item='item' let-index='index' let-depth='depth'>\r\n\r\n    <ng-container [ngTemplateOutlet]='itemTemplate || defaultItemTemplate'\r\n                  [ngTemplateOutletContext]='{item: item, index: index, depth: depth, locale: locale}'>\r\n    </ng-container>\r\n    \r\n    <div *ngIf='item.childrenItemsExpanded'>\r\n      <ng-container *ngFor='let innerItem of (item.childrenItems || []); trackBy: trackById;  let index = index'\r\n                    [ngTemplateOutletContext]='{item: innerItem, index: index, depth: depth + 1, locale: locale}'\r\n                    [ngTemplateOutlet]='itemsIterationTemplate'\r\n      ></ng-container>\r\n    </div>\r\n  </ng-template>\r\n</div>\r\n\r\n<ng-template #defaultItemTemplate let-item='item' let-index='index' let-depth='depth'>\r\n  <div [style.height.px]='rowHeight * (item.streamItems ? item?._streamLevels?.length || 1 : 1)'\r\n       [style.width.px]='width'\r\n       [class.can-collapse]='item.childrenItems && item.childrenItems.length'\r\n       class='panel-item'>\r\n\r\n    <div [style.marginLeft.px]='depth * childGroupOffset' (click)='toggleExpand(item)' class='item-content'>\r\n      <div class='collapse-icon'\r\n           [class.collapsed]='!item.childrenItemsExpanded'>\r\n      </div>\r\n\r\n      <span class='item-name' title=\"{{item.name}}\">{{item.name}}</span>\r\n    </div>\r\n  </div>\r\n</ng-template>\r\n", styles: [":host{display:contents}.panel{height:100%;display:flex;flex-direction:column;position:sticky;left:0;z-index:1001;flex-shrink:0;background-color:#f8f8f8;border-right:1px solid #d0d0d0}.label{display:flex;justify-content:center;align-items:center;width:100%;position:sticky;top:0;left:0;z-index:100;flex-shrink:0;border-bottom:#d0d0d0 1px solid}.panel-item{border-bottom:#d0d0d0 1px solid;padding:5px 10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;box-sizing:border-box;display:flex}.panel-item.can-collapse .collapse-icon{display:inline-block!important}.panel-item.can-collapse .item-content{cursor:pointer}.panel-item .item-content{display:inline-flex;align-items:center;box-sizing:border-box;max-width:100%}.panel-item .item-content .collapse-icon{border:solid #5a5a5a;border-width:0 2px 2px 0;display:none;padding:2.5px;margin-right:8px;transform:rotate(45deg)}.panel-item .item-content .collapse-icon.collapsed{transform:rotate(-45deg)}.panel-item .item-content .item-name{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;width:auto}[mwlResizeHandle]{position:absolute;top:0;right:0;box-sizing:border-box;width:5px;height:100%;cursor:col-resize;z-index:10000}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i2.ResizableDirective, selector: "[mwlResizable]", inputs: ["validateResize", "enableGhostResize", "resizeSnapGrid", "resizeCursors", "ghostElementPositioning", "allowNegativeResizes", "mouseMoveThrottleMS"], outputs: ["resizeStart", "resizing", "resizeEnd"], exportAs: ["mwlResizable"] }, { kind: "directive", type: i2.ResizeHandleDirective, selector: "[mwlResizeHandle]", inputs: ["resizeEdges", "resizableContainer"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelinePanelComponent, decorators: [{
            type: Component,
            args: [{ selector: 'timeline-panel', changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class='panel resize-handle-right'\r\n     mwlResizable\r\n     (resizing)='handleResize($event)'\r\n     [style.width.px]='width'>\r\n\r\n  <div mwlResizeHandle [resizeEdges]=\"{right: resizable}\"></div>\r\n\r\n  <div class='label' [style.height.px]=\"headerHeight\">{{label}}</div>\r\n\r\n  <ng-container *ngFor='let item of items; trackBy: trackById; let index = index'\r\n                [ngTemplateOutlet]='itemsIterationTemplate'\r\n                [ngTemplateOutletContext]='{item: item, index: index, depth: 0}'>\r\n  </ng-container>\r\n\r\n  <ng-template #itemsIterationTemplate let-item='item' let-index='index' let-depth='depth'>\r\n\r\n    <ng-container [ngTemplateOutlet]='itemTemplate || defaultItemTemplate'\r\n                  [ngTemplateOutletContext]='{item: item, index: index, depth: depth, locale: locale}'>\r\n    </ng-container>\r\n    \r\n    <div *ngIf='item.childrenItemsExpanded'>\r\n      <ng-container *ngFor='let innerItem of (item.childrenItems || []); trackBy: trackById;  let index = index'\r\n                    [ngTemplateOutletContext]='{item: innerItem, index: index, depth: depth + 1, locale: locale}'\r\n                    [ngTemplateOutlet]='itemsIterationTemplate'\r\n      ></ng-container>\r\n    </div>\r\n  </ng-template>\r\n</div>\r\n\r\n<ng-template #defaultItemTemplate let-item='item' let-index='index' let-depth='depth'>\r\n  <div [style.height.px]='rowHeight * (item.streamItems ? item?._streamLevels?.length || 1 : 1)'\r\n       [style.width.px]='width'\r\n       [class.can-collapse]='item.childrenItems && item.childrenItems.length'\r\n       class='panel-item'>\r\n\r\n    <div [style.marginLeft.px]='depth * childGroupOffset' (click)='toggleExpand(item)' class='item-content'>\r\n      <div class='collapse-icon'\r\n           [class.collapsed]='!item.childrenItemsExpanded'>\r\n      </div>\r\n\r\n      <span class='item-name' title=\"{{item.name}}\">{{item.name}}</span>\r\n    </div>\r\n  </div>\r\n</ng-template>\r\n", styles: [":host{display:contents}.panel{height:100%;display:flex;flex-direction:column;position:sticky;left:0;z-index:1001;flex-shrink:0;background-color:#f8f8f8;border-right:1px solid #d0d0d0}.label{display:flex;justify-content:center;align-items:center;width:100%;position:sticky;top:0;left:0;z-index:100;flex-shrink:0;border-bottom:#d0d0d0 1px solid}.panel-item{border-bottom:#d0d0d0 1px solid;padding:5px 10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;box-sizing:border-box;display:flex}.panel-item.can-collapse .collapse-icon{display:inline-block!important}.panel-item.can-collapse .item-content{cursor:pointer}.panel-item .item-content{display:inline-flex;align-items:center;box-sizing:border-box;max-width:100%}.panel-item .item-content .collapse-icon{border:solid #5a5a5a;border-width:0 2px 2px 0;display:none;padding:2.5px;margin-right:8px;transform:rotate(45deg)}.panel-item .item-content .collapse-icon.collapsed{transform:rotate(-45deg)}.panel-item .item-content .item-name{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;width:auto}[mwlResizeHandle]{position:absolute;top:0;right:0;box-sizing:border-box;width:5px;height:100%;cursor:col-resize;z-index:10000}\n"] }]
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }], propDecorators: { items: [{
                type: Input
            }], label: [{
                type: Input
            }], width: [{
                type: Input
            }], resizable: [{
                type: Input
            }], minWidth: [{
                type: Input
            }], maxWidth: [{
                type: Input
            }], headerHeight: [{
                type: Input
            }], rowHeight: [{
                type: Input
            }], locale: [{
                type: Input
            }], childGroupOffset: [{
                type: Input
            }], itemTemplate: [{
                type: Input
            }], widthChanged: [{
                type: Output
            }] } });

class TimelineComponent {
    /**
     * Register array of custom zooms.
     * Current zoom can be changed to any existed in this array by calling method "changeZoom()"
     */
    set zooms(value) {
        this.zoomsHandler.setZooms(value);
    }
    /**
     * The items of timeline.
     */
    set items(items) {
        this.itemsIterator.setItems(items);
        this.redraw();
    }
    /**
     * Visible timeline width (container visible width - panel width = timeline visible width).
     */
    get visibleScaleWidth() {
        return this._elementRef.nativeElement.clientWidth - this.panelWidth;
    }
    /**
     * Active zoom.
     */
    get zoom() {
        return this.zoomsHandler.activeZoom;
    }
    /**
     * Registered zooms list.
     */
    get zooms() {
        return this.zoomsHandler.zooms;
    }
    constructor(_cdr, _strategyManager, _elementRef, _platformId) {
        this._cdr = _cdr;
        this._strategyManager = _strategyManager;
        this._elementRef = _elementRef;
        this._platformId = _platformId;
        /**
         * Indicates the current shown date in the middle of user`s screen.
         */
        this.currentDate = new Date();
        this.dateMarkerLeftPosition = 0;
        this.itemsIterator = new ItemsIterator();
        this.zoomsHandler = new ZoomsHandler(DefaultZooms);
        this._ignoreNextScrollEvent = false;
        this._destroy$ = new Subject();
        /**
         * Emits event when startDate and endDate of some item was changed by resizing/moving it.
         */
        this.itemTimeChanged = new EventEmitter();
        /**
         * Emits event when item was moved by Y axis.
         */
        this.itemRowChanged = new EventEmitter();
        /**
         * Emits event when current zoom was changed.
         */
        this.zoomChanged = new EventEmitter();
        /**
         * Emits event when user clicked somewhere on time grid.
         */
        this.timeGridClicked = new EventEmitter();
        /**
         * The locale used to format dates. By default is 'en'
         */
        this.locale = 'en';
        /**
         * Height of the each row in pixels. By default is 40.
         */
        this.rowHeight = 40;
        /**
         * Height of the each timeline item in pixels. Can't be bigger then 'rowHeight' property. By default is 30.
         */
        this.itemHeight = 30;
        /**
         * Height of top dates panel in pixels. By default is 60.
         */
        this.headerHeight = 60;
        /**
         * The label of left panel. By default is empty.
         */
        this.panelLabel = '';
        /**
         * Width of left panel in pixels. By default is 160.
         */
        this.panelWidth = 160;
        /**
         * Minimal width of left panel in pixels. By default is 50.
         */
        this.minPanelWidth = 50;
        /**
         * Maximal width of left panel in pixels. By default is 400.
         */
        this.maxPanelWidth = 400;
        /**
         *  Sets the left displacement in pixels between parent and child groups in left panel. By default is 15.
         */
        this.offsetForChildPanelItem = 15;
        /**
         * Can resize panel. By default is true.
         */
        this.isPanelResizable = true;
        /**
         * If false then date marker will be not visible.
         */
        this.showDateMarket = true;
        this._setStrategies(this.zoom);
    }
    ngAfterViewInit() {
        this.zoomsHandler.activeZoom$
            .pipe(takeUntil(this._destroy$))
            .subscribe((zoom) => {
            this._setStrategies(zoom);
            this.redraw();
            this.zoomChanged.emit(zoom);
        });
        if (isPlatformBrowser(this._platformId)) {
            interval(MillisecondsToTime.Minute)
                .pipe(startWith(''), takeUntil(this._destroy$))
                .subscribe(() => this._recalculateDateMarkerPosition());
        }
    }
    /**
     * Recalculate and update view.
     */
    redraw() {
        this._generateScale();
        this._updateItemsPosition();
        this.itemsIterator.setItems([...this.itemsIterator.items]);
        this._recalculateDateMarkerPosition();
        this._ignoreNextScrollEvent = true;
        this._cdr.detectChanges();
        this.attachCameraToDate(this.currentDate);
    }
    /**
     * Set horizontal scroll in the middle of the date
     */
    attachCameraToDate(date) {
        this.currentDate = date;
        const duration = this.viewModeAdaptor.getDurationInColumns(this.scale.startDate, date);
        const scrollLeft = (duration * this.zoom.columnWidth) - (this.visibleScaleWidth / 2);
        this._ignoreNextScrollEvent = true;
        if (this._elementRef.nativeElement) {
            this._elementRef.nativeElement.scrollLeft = scrollLeft < 0 ? 0 : scrollLeft;
        }
    }
    /**
     * Automatically chooses the most optimal zoom and sets horizontal scroll to the center of the items.
     * Padding sets minimal spacing from left and right to the first and last items.
     */
    fitToContent(paddings) {
        const firstItem = this.itemsIterator.getFirstItem(true);
        const lastItem = this.itemsIterator.getLastItem(true);
        if (!firstItem || !lastItem)
            return;
        const startDate = new Date(firstItem.startDate);
        const endDate = new Date(lastItem.endDate);
        const zoom = this._calculateOptimalZoom(startDate, endDate, paddings);
        const viewModeAdaptor = this._strategyManager.getViewModeAdaptor(zoom.viewMode);
        this.currentDate = new Date(viewModeAdaptor.getMiddleDate(startDate, endDate));
        if (this.zoomsHandler.isZoomActive(zoom)) {
            this.attachCameraToDate(this.currentDate);
        }
        else {
            this.changeZoom(zoom);
        }
    }
    /**
     * Change zoom to one of the existed
     */
    changeZoom(zoom) {
        this.zoomsHandler.changeActiveZoom(zoom);
    }
    /**
     * Find zoom by its index and change
     */
    changeZoomByIndex(index) {
        const zoom = this.zoomsHandler.zooms.find(zoom => zoom.index === index);
        if (zoom) {
            this.zoomsHandler.changeActiveZoom(zoom);
        }
        else {
            console.error(`Cannot find zoom with index ${index}`);
        }
    }
    /**
     * Changes zoom to the max value
     */
    zoomFullIn() {
        this.zoomsHandler.changeActiveZoom(this.zoomsHandler.getLastZoom());
    }
    /**
     * Changes zoom to the min value
     */
    zoomFullOut() {
        this.zoomsHandler.changeActiveZoom(this.zoomsHandler.getFirstZoom());
    }
    /**
     * Changes zoom for 1 step back
     */
    zoomIn() {
        this.zoomsHandler.zoomIn();
    }
    /**
     * Changes zoom for 1 step forward
     */
    zoomOut() {
        this.zoomsHandler.zoomOut();
    }
    /**
     * Accepts the relative coordinates to the timeline container and returns the row and column.
     */
    getCellByCoordinates(x, y) {
        const rowDeterminant = new RowDeterminant(this.itemsIterator);
        const rowIndex = Math.floor((y - this.headerHeight) / this.rowHeight);
        const row = rowDeterminant.getStreamByRowIndex(rowIndex);
        const columnIndex = Math.floor((x - this.panelWidth) / this.zoom.columnWidth);
        const column = this.scale.columns[columnIndex];
        return { column, row };
    }
    _getCurrentDate() {
        const currentScrollLeft = this._elementRef.nativeElement.scrollLeft ?? 0;
        const scrollLeftToCenterScreen = currentScrollLeft + (this.visibleScaleWidth / 2);
        const columns = Math.round(scrollLeftToCenterScreen / this.zoom.columnWidth);
        return this.viewModeAdaptor.addColumnToDate(this.scale.startDate, columns);
    }
    _onItemMoved(event, item) {
        if (event.y) {
            this._onItemMovedVertically(event, item);
        }
        if (event.x) {
            this._onItemMovedHorizontally(event, item);
        }
    }
    _onItemMovedHorizontally(event, item) {
        const transferColumns = Math.round(event.x / this.zoom.columnWidth);
        const newStartDate = this.viewModeAdaptor.addColumnToDate(new Date(item.startDate), transferColumns);
        const newEndDate = this.viewModeAdaptor.addColumnToDate(new Date(item.endDate), transferColumns);
        this.itemTimeChanged.emit({ item, newStartDate, newEndDate });
    }
    _onItemMovedVertically(event, item) {
        const rowDeterminant = new RowDeterminant(this.itemsIterator);
        const rowIndex = rowDeterminant.getRowIndexByItem(item);
        const transferRows = event.y / this.rowHeight;
        const newRowIndex = rowIndex + transferRows;
        if (rowIndex === newRowIndex)
            return;
        const oldRow = rowDeterminant.getStreamByRowIndex(rowIndex);
        const newRow = rowDeterminant.getStreamByRowIndex(newRowIndex);
        this.itemRowChanged.emit({ item, oldRow, newRow });
    }
    _calculateOptimalZoom(startDate, endDate, paddings = 15) {
        let possibleZoom = this.zoomsHandler.getFirstZoom();
        for (let i = this.zoomsHandler.getLastZoom().index; i >= this.zoomsHandler.getFirstZoom().index; i--) {
            const currentZoom = this.zoomsHandler.zooms[i];
            const viewModeAdaptor = this._strategyManager.getViewModeAdaptor(currentZoom.viewMode);
            const countOfColumns = viewModeAdaptor.getUniqueColumnsWithinRange(startDate, endDate);
            if (countOfColumns * currentZoom.columnWidth < (this.visibleScaleWidth - paddings * 2)) {
                possibleZoom = currentZoom;
                break;
            }
        }
        return possibleZoom;
    }
    _trackById(index, item) {
        return item.id;
    }
    _handleContentClick(event) {
        const scrollLeft = this._elementRef.nativeElement.scrollLeft;
        const scrollTop = this._elementRef.nativeElement.scrollTop;
        const rect = this._elementRef.nativeElement.getBoundingClientRect();
        const xClick = event.clientX - rect.left + scrollLeft;
        const yClick = event.clientY - rect.top + scrollTop;
        const cell = this.getCellByCoordinates(xClick, yClick);
        this.timeGridClicked.emit({ originalEvent: event, column: cell.column, row: cell.row });
    }
    _onItemResized(event, item) {
        const calculateNewDate = (movedPx, oldDate) => {
            const countOfColumnsMoved = Math.round(movedPx / this.zoom.columnWidth);
            return this.viewModeAdaptor.addColumnToDate(oldDate, countOfColumnsMoved);
        };
        if (event.edges.left) {
            const newStartDate = calculateNewDate(event.edges.left, new Date(item.startDate));
            const isNewStartDateValid = this.viewModeAdaptor.getBeginningDateOfColumn(newStartDate).getTime() <= new Date(item.endDate).getTime();
            if (isNewStartDateValid) {
                this.itemTimeChanged.emit({ item, newStartDate });
            }
        }
        else {
            const newEndDate = calculateNewDate(event.edges.right, new Date(item.endDate));
            const isNewEndDateValid = this.viewModeAdaptor.getEndingDateOfColumn(newEndDate).getTime() >= new Date(item.startDate).getTime();
            if (isNewEndDateValid) {
                this.itemTimeChanged.emit({ item, newEndDate });
            }
        }
    }
    _onScroll(event) {
        if (!this._ignoreNextScrollEvent) {
            this.currentDate = this._getCurrentDate();
        }
        this._ignoreNextScrollEvent = false;
    }
    _generateScale() {
        const scaleStartDate = this.scaleGenerator.getStartDate(this.itemsIterator);
        const scaleEndDate = this.scaleGenerator.getEndDate(this.itemsIterator);
        this.scale = this.scaleGenerator.generateScale(scaleStartDate, scaleEndDate);
    }
    _updateItemsPosition() {
        this.itemsIterator.forEach((item) => this._updateItemPosition(item));
    }
    _updateItemPosition(item) {
        item._width = this._calculateItemWidth(item);
        item._left = this._calculateItemLeftPosition(item);
        item.updateView && item.updateView();
    }
    _calculateItemLeftPosition(item) {
        if (!item.startDate || !item.endDate)
            return 0;
        const columnsOffsetFromStart = this.viewModeAdaptor.getUniqueColumnsWithinRange(this.scale.startDate, new Date(item.startDate)) - 1;
        return columnsOffsetFromStart * this.zoom.columnWidth;
    }
    _calculateItemWidth(item) {
        if (!item.startDate || !item.endDate)
            return 0;
        const columnsOccupied = this.viewModeAdaptor.getUniqueColumnsWithinRange(new Date(item.startDate), new Date(item.endDate));
        return columnsOccupied * this.zoom.columnWidth;
    }
    _recalculateDateMarkerPosition() {
        const countOfColumns = this.viewModeAdaptor.getDurationInColumns(this.scale.startDate, new Date());
        this.dateMarkerLeftPosition = countOfColumns * this.zoom.columnWidth;
    }
    _setStrategies(zoom) {
        this.viewModeAdaptor = this._strategyManager.getViewModeAdaptor(zoom.viewMode);
        this.scaleGenerator = this._strategyManager.getScaleGenerator(zoom.viewMode);
    }
    ngOnDestroy() {
        this._destroy$.next();
        this._destroy$.complete();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineComponent, deps: [{ token: i0.ChangeDetectorRef }, { token: StrategyManager }, { token: ElementRef }, { token: PLATFORM_ID }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: TimelineComponent, selector: "timeline-calendar", inputs: { locale: "locale", rowHeight: "rowHeight", itemHeight: "itemHeight", headerHeight: "headerHeight", panelLabel: "panelLabel", panelWidth: "panelWidth", minPanelWidth: "minPanelWidth", maxPanelWidth: "maxPanelWidth", offsetForChildPanelItem: "offsetForChildPanelItem", isPanelResizable: "isPanelResizable", showDateMarket: "showDateMarket", panelItemTemplate: "panelItemTemplate", itemContentTemplate: "itemContentTemplate", columnSeparatorTemplate: "columnSeparatorTemplate", dateMarkerTemplate: "dateMarkerTemplate", zooms: "zooms", items: "items" }, outputs: { itemTimeChanged: "itemTimeChanged", itemRowChanged: "itemRowChanged", zoomChanged: "zoomChanged", timeGridClicked: "timeGridClicked" }, host: { listeners: { "scroll": "_onScroll($event)" } }, ngImport: i0, template: "<timeline-panel\r\n  (widthChanged)=\"panelWidth = $event\"\r\n  [width]=\"panelWidth\"\r\n  [maxWidth]=\"maxPanelWidth\"\r\n  [minWidth]=\"minPanelWidth\"\r\n  [resizable]=\"isPanelResizable\"\r\n  [itemTemplate]=\"panelItemTemplate\"\r\n  [label]=\"panelLabel\"\r\n  [headerHeight]=\"headerHeight\"\r\n  [items]=\"itemsIterator.items\"\r\n  [locale]=\"locale\"\r\n  [childGroupOffset]=\"offsetForChildPanelItem\"\r\n  [rowHeight]=\"rowHeight\">\r\n</timeline-panel>\r\n\r\n<div class='content-wrapper' (click)=\"_handleContentClick($event)\">\r\n  <timeline-scale-header\r\n    [height]=\"headerHeight\"\r\n    [formatter]=\"scaleGenerator.formatter\"\r\n    [zoom]='zoom'\r\n    [scale]='scale'\r\n    [locale]=\"locale\"\r\n  ></timeline-scale-header>\r\n\r\n  <div class=\"column-separators\">\r\n    <ng-container *ngFor='let date of scale?.columns; index as i; trackBy: _trackById'>\r\n      <div class=\"line\" *ngIf=\"!columnSeparatorTemplate\"\r\n           [style.left.px]='zoom.columnWidth * (i + 1)'\r\n           [style.height]=\"'calc(100% - ' + headerHeight + 'px)'\">\r\n      </div>\r\n\r\n      <ng-template [ngTemplateOutlet]=\"columnSeparatorTemplate\"\r\n                   [ngTemplateOutletContext]=\"{column: date, index: i, columnWidth: zoom.columnWidth, headerHeight: headerHeight}\">\r\n      </ng-template>\r\n    </ng-container>\r\n  </div>\r\n\r\n  <timeline-date-marker *ngIf=\"showDateMarket\"\r\n                        [leftPosition]=\"dateMarkerLeftPosition\"\r\n                        [headerHeight]=\"headerHeight\"\r\n                        [scale]=\"scale\"\r\n                        [customTemplate]=\"dateMarkerTemplate\"\r\n  ></timeline-date-marker>\r\n\r\n  <div class='timeline-items' *ngIf=\"!itemsIterator.isEmpty()\">\r\n    <ng-container *ngFor='let group of itemsIterator.items; trackBy: _trackById;'>\r\n      <ng-container *ngIf=\"!group.streamItems\">\r\n        <div [style.height.px]='rowHeight' #rowContainer class='item-row'>\r\n          <ng-container [ngTemplateOutlet]='itemTemplate'\r\n                        [ngTemplateOutletContext]='{$implicit: group, rowContainer: rowContainer}'\r\n          ></ng-container>\r\n        </div>\r\n      </ng-container>\r\n\r\n      <ng-container [ngTemplateOutlet]='itemsIterationTemplate'\r\n                    [ngTemplateOutletContext]='{$implicit: group}'\r\n      ></ng-container>\r\n    </ng-container>\r\n\r\n    <ng-template #itemTemplate let-item let-rowContainer=\"rowContainer\">\r\n      <timeline-item [item]='item'\r\n                     [rowContainer]='rowContainer'\r\n                     [scale]=\"scale\"\r\n                     [height]=\"itemHeight\"\r\n                     [rowHeight]=\"rowHeight\"\r\n                     [locale]=\"locale\"\r\n                     (itemResized)=\"_onItemResized($event.event, $event.item)\"\r\n                     (itemMoved)=\"_onItemMoved($event.event, $event.item)\"\r\n                     [contentTemplate]='itemContentTemplate'\r\n      ></timeline-item>\r\n    </ng-template>\r\n\r\n    <ng-template #itemsIterationTemplate let-group>\r\n        <div *ngFor='let streamLevel of (group._streamLevels || []); trackBy: _trackById;'>\r\n          <div [style.height.px]='rowHeight' class='item-row' #rowContainer>\r\n            <ng-container *ngFor=\"let item of streamLevel\">\r\n              <ng-container [ngTemplateOutlet]=\"itemTemplate\"\r\n                            [ngTemplateOutletContext]=\"{$implicit: item, rowContainer: rowContainer}\"\r\n              ></ng-container>\r\n            </ng-container>\r\n          </div>\r\n        </div>\r\n\r\n      <div [style.height.px]='rowHeight' class='item-row' *ngIf=\"group?._streamLevels?.length === 0\">\r\n      </div>\r\n\r\n      <ng-container *ngIf='group.childrenItemsExpanded'>\r\n        <div *ngFor='let item of (group.childrenItems || []); trackBy: _trackById;'>\r\n          <div [style.height.px]='rowHeight' *ngIf=\"!item.streamItems\" class='item-row' #rowContainer>\r\n            <ng-container [ngTemplateOutlet]=\"itemTemplate\"\r\n                          [ngTemplateOutletContext]=\"{$implicit: item, rowContainer: rowContainer}\"\r\n            ></ng-container>\r\n          </div>\r\n\r\n          <ng-container [ngTemplateOutlet]='itemsIterationTemplate'\r\n                        [ngTemplateOutletContext]='{$implicit: item}'\r\n          ></ng-container>\r\n        </div>\r\n      </ng-container>\r\n    </ng-template>\r\n  </div>\r\n</div>\r\n", styles: [":host{height:100%;overflow:auto;display:flex;flex-grow:1;flex-direction:row;font-size:14px;position:relative}.content-wrapper{position:relative;height:fit-content;min-height:100%}.column-separators .line{position:absolute;width:1px;height:100%;background-color:#d0d0d0}.timeline-items{display:flex;flex-direction:column;height:auto}.timeline-items .item-row{display:flex;align-items:center;flex-direction:row;position:relative;z-index:7;box-sizing:border-box}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: TimelineItemComponent, selector: "timeline-item", inputs: ["item", "scale", "rowContainer", "height", "rowHeight", "locale", "contentTemplate"], outputs: ["itemResized", "itemMoved"] }, { kind: "component", type: TimelineDateMarkerComponent, selector: "timeline-date-marker", inputs: ["leftPosition", "headerHeight", "customTemplate", "scale"] }, { kind: "component", type: TimelineScaleHeaderComponent, selector: "timeline-scale-header", inputs: ["height", "scale", "formatter", "locale", "zoom"] }, { kind: "component", type: TimelinePanelComponent, selector: "timeline-panel", inputs: ["items", "label", "width", "resizable", "minWidth", "maxWidth", "headerHeight", "rowHeight", "locale", "childGroupOffset", "itemTemplate"], outputs: ["widthChanged"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineComponent, decorators: [{
            type: Component,
            args: [{ selector: 'timeline-calendar', changeDetection: ChangeDetectionStrategy.OnPush, template: "<timeline-panel\r\n  (widthChanged)=\"panelWidth = $event\"\r\n  [width]=\"panelWidth\"\r\n  [maxWidth]=\"maxPanelWidth\"\r\n  [minWidth]=\"minPanelWidth\"\r\n  [resizable]=\"isPanelResizable\"\r\n  [itemTemplate]=\"panelItemTemplate\"\r\n  [label]=\"panelLabel\"\r\n  [headerHeight]=\"headerHeight\"\r\n  [items]=\"itemsIterator.items\"\r\n  [locale]=\"locale\"\r\n  [childGroupOffset]=\"offsetForChildPanelItem\"\r\n  [rowHeight]=\"rowHeight\">\r\n</timeline-panel>\r\n\r\n<div class='content-wrapper' (click)=\"_handleContentClick($event)\">\r\n  <timeline-scale-header\r\n    [height]=\"headerHeight\"\r\n    [formatter]=\"scaleGenerator.formatter\"\r\n    [zoom]='zoom'\r\n    [scale]='scale'\r\n    [locale]=\"locale\"\r\n  ></timeline-scale-header>\r\n\r\n  <div class=\"column-separators\">\r\n    <ng-container *ngFor='let date of scale?.columns; index as i; trackBy: _trackById'>\r\n      <div class=\"line\" *ngIf=\"!columnSeparatorTemplate\"\r\n           [style.left.px]='zoom.columnWidth * (i + 1)'\r\n           [style.height]=\"'calc(100% - ' + headerHeight + 'px)'\">\r\n      </div>\r\n\r\n      <ng-template [ngTemplateOutlet]=\"columnSeparatorTemplate\"\r\n                   [ngTemplateOutletContext]=\"{column: date, index: i, columnWidth: zoom.columnWidth, headerHeight: headerHeight}\">\r\n      </ng-template>\r\n    </ng-container>\r\n  </div>\r\n\r\n  <timeline-date-marker *ngIf=\"showDateMarket\"\r\n                        [leftPosition]=\"dateMarkerLeftPosition\"\r\n                        [headerHeight]=\"headerHeight\"\r\n                        [scale]=\"scale\"\r\n                        [customTemplate]=\"dateMarkerTemplate\"\r\n  ></timeline-date-marker>\r\n\r\n  <div class='timeline-items' *ngIf=\"!itemsIterator.isEmpty()\">\r\n    <ng-container *ngFor='let group of itemsIterator.items; trackBy: _trackById;'>\r\n      <ng-container *ngIf=\"!group.streamItems\">\r\n        <div [style.height.px]='rowHeight' #rowContainer class='item-row'>\r\n          <ng-container [ngTemplateOutlet]='itemTemplate'\r\n                        [ngTemplateOutletContext]='{$implicit: group, rowContainer: rowContainer}'\r\n          ></ng-container>\r\n        </div>\r\n      </ng-container>\r\n\r\n      <ng-container [ngTemplateOutlet]='itemsIterationTemplate'\r\n                    [ngTemplateOutletContext]='{$implicit: group}'\r\n      ></ng-container>\r\n    </ng-container>\r\n\r\n    <ng-template #itemTemplate let-item let-rowContainer=\"rowContainer\">\r\n      <timeline-item [item]='item'\r\n                     [rowContainer]='rowContainer'\r\n                     [scale]=\"scale\"\r\n                     [height]=\"itemHeight\"\r\n                     [rowHeight]=\"rowHeight\"\r\n                     [locale]=\"locale\"\r\n                     (itemResized)=\"_onItemResized($event.event, $event.item)\"\r\n                     (itemMoved)=\"_onItemMoved($event.event, $event.item)\"\r\n                     [contentTemplate]='itemContentTemplate'\r\n      ></timeline-item>\r\n    </ng-template>\r\n\r\n    <ng-template #itemsIterationTemplate let-group>\r\n        <div *ngFor='let streamLevel of (group._streamLevels || []); trackBy: _trackById;'>\r\n          <div [style.height.px]='rowHeight' class='item-row' #rowContainer>\r\n            <ng-container *ngFor=\"let item of streamLevel\">\r\n              <ng-container [ngTemplateOutlet]=\"itemTemplate\"\r\n                            [ngTemplateOutletContext]=\"{$implicit: item, rowContainer: rowContainer}\"\r\n              ></ng-container>\r\n            </ng-container>\r\n          </div>\r\n        </div>\r\n\r\n      <div [style.height.px]='rowHeight' class='item-row' *ngIf=\"group?._streamLevels?.length === 0\">\r\n      </div>\r\n\r\n      <ng-container *ngIf='group.childrenItemsExpanded'>\r\n        <div *ngFor='let item of (group.childrenItems || []); trackBy: _trackById;'>\r\n          <div [style.height.px]='rowHeight' *ngIf=\"!item.streamItems\" class='item-row' #rowContainer>\r\n            <ng-container [ngTemplateOutlet]=\"itemTemplate\"\r\n                          [ngTemplateOutletContext]=\"{$implicit: item, rowContainer: rowContainer}\"\r\n            ></ng-container>\r\n          </div>\r\n\r\n          <ng-container [ngTemplateOutlet]='itemsIterationTemplate'\r\n                        [ngTemplateOutletContext]='{$implicit: item}'\r\n          ></ng-container>\r\n        </div>\r\n      </ng-container>\r\n    </ng-template>\r\n  </div>\r\n</div>\r\n", styles: [":host{height:100%;overflow:auto;display:flex;flex-grow:1;flex-direction:row;font-size:14px;position:relative}.content-wrapper{position:relative;height:fit-content;min-height:100%}.column-separators .line{position:absolute;width:1px;height:100%;background-color:#d0d0d0}.timeline-items{display:flex;flex-direction:column;height:auto}.timeline-items .item-row{display:flex;align-items:center;flex-direction:row;position:relative;z-index:7;box-sizing:border-box}\n"] }]
        }], ctorParameters: () => [{ type: i0.ChangeDetectorRef }, { type: StrategyManager }, { type: i0.ElementRef, decorators: [{
                    type: Inject,
                    args: [ElementRef]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }], propDecorators: { itemTimeChanged: [{
                type: Output
            }], itemRowChanged: [{
                type: Output
            }], zoomChanged: [{
                type: Output
            }], timeGridClicked: [{
                type: Output
            }], locale: [{
                type: Input
            }], rowHeight: [{
                type: Input
            }], itemHeight: [{
                type: Input
            }], headerHeight: [{
                type: Input
            }], panelLabel: [{
                type: Input
            }], panelWidth: [{
                type: Input
            }], minPanelWidth: [{
                type: Input
            }], maxPanelWidth: [{
                type: Input
            }], offsetForChildPanelItem: [{
                type: Input
            }], isPanelResizable: [{
                type: Input
            }], showDateMarket: [{
                type: Input
            }], panelItemTemplate: [{
                type: Input
            }], itemContentTemplate: [{
                type: Input
            }], columnSeparatorTemplate: [{
                type: Input
            }], dateMarkerTemplate: [{
                type: Input
            }], zooms: [{
                type: Input
            }], items: [{
                type: Input
            }], _onScroll: [{
                type: HostListener,
                args: ['scroll', ['$event']]
            }] } });

class TimelineModule {
    static forChild(config) {
        return {
            ngModule: TimelineModule,
            providers: [
                config?.strategyManager ?? StrategyManager,
                config?.dayScaleGenerator ?? DayScaleGenerator,
                config?.weekScaleGenerator ?? WeekScaleGenerator,
                config?.monthScaleGenerator ?? MonthScaleGenerator,
                {
                    provide: DAY_SCALE_GENERATOR_CONFIG,
                    useValue: config?.dayScaleConfig
                },
                {
                    provide: WEEK_SCALE_GENERATOR_CONFIG,
                    useValue: config?.weekScaleConfig
                },
                {
                    provide: MONTH_SCALE_GENERATOR_CONFIG,
                    useValue: config?.monthScaleConfig
                },
            ]
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.13", ngImport: i0, type: TimelineModule, declarations: [TimelineComponent,
            TimelineItemComponent,
            TimelineDateMarkerComponent,
            TimelineScaleHeaderComponent,
            TimelinePanelComponent], imports: [CommonModule,
            ResizableModule,
            DragAndDropModule], exports: [TimelineComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineModule, imports: [CommonModule,
            ResizableModule,
            DragAndDropModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TimelineModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TimelineComponent,
                        TimelineItemComponent,
                        TimelineDateMarkerComponent,
                        TimelineScaleHeaderComponent,
                        TimelinePanelComponent
                    ],
                    imports: [
                        CommonModule,
                        ResizableModule,
                        DragAndDropModule,
                    ],
                    exports: [
                        TimelineComponent,
                    ],
                }]
        }] });

/*
 * Public API Surface of angular-calendar-timeline
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DayScaleGenerator, DefaultStrategyManager, DefaultZooms, MonthScaleGenerator, StrategyManager, TimelineComponent, TimelineModule, TimelineViewMode, WeekScaleGenerator };
//# sourceMappingURL=angular-calendar-timeline.mjs.map
